package com.persagy.apm.diagnose.indicatorcompute.service.impl;

import com.persagy.apm.diagnose.indicatorcompute.dao.ProjectIndicatorComputeMapper;
import com.persagy.apm.diagnose.indicatorcompute.service.IProjectIndicatorComputeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.diagnose.indicatorcompute.model.*;
import com.persagy.apm.diagnose.indicatorcompute.model.dto.*;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * (ProjectIndicatorCompute) service层
 *
 * @author lixing
 * @version V1.0 2021-09-21 13:32:04
 */
@Service
public class ProjectIndicatorComputeServiceImpl extends ServiceImpl<ProjectIndicatorComputeMapper, ProjectIndicatorCompute> 
    implements IProjectIndicatorComputeService {
    
    /**
    * 创建
    * @return 主键
    * @author lixing
    * @version V1.0 2021-09-21 13:32:04
    */
    @Override
    public String createProjectIndicatorCompute(AddProjectIndicatorComputeDTO addProjectIndicatorComputeDTO) {
        ProjectIndicatorCompute projectIndicatorCompute = ConvertProjectIndicatorComputeTool.INSTANCE.convertAddDto2Entity(addProjectIndicatorComputeDTO);
        save(projectIndicatorCompute);
        return null;
    }

    /**
    * 详情
    * @param id 主键
    * @return 部门do类
    * @author lixing
    * @version V1.0 2021-09-21 13:32:04
    */
    @Override
    public ProjectIndicatorCompute queryProjectIndicatorComputeDetail(String id) {
        ProjectIndicatorCompute projectIndicatorCompute = getById(id);
        if (projectIndicatorCompute == null) {
            throw new IllegalArgumentException("查看ProjectIndicatorCompute详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return projectIndicatorCompute;
    }
    
    /**
    * 更新
    * @author lixing
    * @version V1.0 2021-09-21 13:32:04
    */
    @Override
    public void updateProjectIndicatorCompute(UpdateProjectIndicatorComputeDTO updateProjectIndicatorComputeDTO) {
        ProjectIndicatorCompute projectIndicatorCompute = getById(updateProjectIndicatorComputeDTO.getId());
        projectIndicatorCompute = ConvertProjectIndicatorComputeTool.INSTANCE.convertUpdateDto2Entity(projectIndicatorCompute, updateProjectIndicatorComputeDTO);
        updateById(projectIndicatorCompute);
    }
    
    /**
     * 校验是否可删除
     *
     * @param id 主键
     * @return do类
     * @author lixing
     * @version V1.0 2021-09-21 13:32:04
     */
    public ProjectIndicatorCompute checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ProjectIndicatorCompute时发生异常，主键为空");
        }

        ProjectIndicatorCompute projectIndicatorCompute = getById(id);

        if (projectIndicatorCompute == null) {
            throw new IllegalArgumentException("删除ProjectIndicatorCompute时发生异常，找不到要删除的数据，id:" + id);
        }

        return projectIndicatorCompute;
    }
    
    /**
    * 删除
    * @param id 主键
    * @author lixing
    * @version V1.0 2021-09-21 13:32:04
    */
    @Override
    public void deleteProjectIndicatorCompute(String id) {
        // 校验是否可删除
        ProjectIndicatorCompute projectIndicatorCompute = checkDeletable(id);
        updateById(projectIndicatorCompute);
    }
    
    /**
    * 查询
    * @return List<ProjectIndicatorCompute>
    * @author lixing
    * @version V1.0 2021-09-21 13:32:04
    */
    @Override
    public List<ProjectIndicatorCompute> queryProjectIndicatorComputeList(QueryProjectIndicatorComputeDTO queryProjectIndicatorComputeDTO) {
        QueryWrapper<ProjectIndicatorCompute> queryWrapper = new QueryWrapper<>();
        // 默认按创建时间倒序排序
        if (queryProjectIndicatorComputeDTO != null) {
                                
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectIndicatorComputeDTO.getProjectId())) {
                queryWrapper.like(ProjectIndicatorCompute.PROP_PROJECT_ID, queryProjectIndicatorComputeDTO.getProjectId());
            }
                                            
            if (queryProjectIndicatorComputeDTO.getComputeRound() != null) {
                queryWrapper.eq(ProjectIndicatorCompute.PROP_COMPUTE_ROUND, queryProjectIndicatorComputeDTO.getComputeRound());
            }
                                            
            if (queryProjectIndicatorComputeDTO.getComputeTime() != null) {
                queryWrapper.eq(ProjectIndicatorCompute.PROP_COMPUTE_TIME, queryProjectIndicatorComputeDTO.getComputeTime());
            }
                                            
        }
        
        return list(queryWrapper);
    }
    
    /**
    * 分页查询
    * @return IPage<ProjectIndicatorCompute>
    * @author lixing
    * @version V1.0 2021-09-21 13:32:04
    */
    @Override
    public IPage<ProjectIndicatorCompute> pageQueryProjectIndicatorCompute(PageQueryProjectIndicatorComputeDTO pageQueryProjectIndicatorComputeDTO) {
        QueryWrapper<ProjectIndicatorCompute> queryWrapper = new QueryWrapper<>();
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ProjectIndicatorCompute> pageParam = new Page<>(pageQueryProjectIndicatorComputeDTO.getPage(), pageQueryProjectIndicatorComputeDTO.getSize(),true);
        // 排序信息
        if(CollectionUtils.isEmpty(pageQueryProjectIndicatorComputeDTO.getOrders())){
            // 默认按创建时间倒序排序
        }else {
            List<Sort> orders = pageQueryProjectIndicatorComputeDTO.getOrders();
            for(Sort sort: orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE,sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }
                
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectIndicatorComputeDTO.getProjectId())) {
            queryWrapper.like(ProjectIndicatorCompute.PROP_PROJECT_ID, pageQueryProjectIndicatorComputeDTO.getProjectId());
        }
                            
        if (pageQueryProjectIndicatorComputeDTO.getComputeRound() != null) {
            queryWrapper.eq(ProjectIndicatorCompute.PROP_COMPUTE_ROUND, pageQueryProjectIndicatorComputeDTO.getComputeRound());
        }
                            
        if (pageQueryProjectIndicatorComputeDTO.getComputeTime() != null) {
            queryWrapper.eq(ProjectIndicatorCompute.PROP_COMPUTE_TIME, pageQueryProjectIndicatorComputeDTO.getComputeTime());
        }
                                    
        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public Map<String, ProjectIndicatorCompute> getProjectIdAndProjectCompute() {
        List<ProjectIndicatorCompute> projectIndicatorComputes = this.queryProjectIndicatorComputeList(new QueryProjectIndicatorComputeDTO());
        return projectIndicatorComputes.stream().collect(Collectors.toMap(ProjectIndicatorCompute::getProjectId, Function.identity(),(key1, key2)->key2));
    }

}
